
#include "html_tools.h"

void responseClient()
{
  WiFiClient client = server.available();
  if (!client)
  {
    return;
  }
  // Wait until the client sends some data
  Serial.println("new client");
  unsigned long ultimeout = millis() + 250;
  while (!client.available() && (millis() < ultimeout) )
  {
    yield();
  }
  if (millis() > ultimeout)
  {
    Serial.println("client connection time-out!");
    return;
  }

  // Read the first line of the request
  String sRequest = client.readStringUntil('\r');
  client.flush();

  // stop client, if request is empty
  if (sRequest == "")
  {
    Serial.println("empty request! - stopping client");
    client.stop();
    return;
  }

#ifdef HTML_DEBUG
  Serial.println("Clientdaten");
  Serial.println(sRequest);
  Serial.println("Clientdaten Ende");
#endif

  decodeParameter(sRequest);                          // Parameterliste in pageDaten schreiben

#ifdef HTML_DEBUG
  Serial.println("Parameter");
  Serial.print("Pfad: ");
  Serial.println(pageDaten.Path);
  for (int i=0; i<MAX_PARAMETER; i++)
  {
    if (i<10)
    {
      Serial.print(" ");     
    }
    Serial.print(i);
    Serial.print(" - ");     
    Serial.print(pageDaten.Index[i]);
    Serial.print(" # ");
    Serial.println(pageDaten.Value[i]);    
  }
  Serial.println("");

  Serial.println("Ersetzungen");
  for (int i=0; i<ANZ_REPLACE; i++)
  {
    if (i<10)
    {
      Serial.print(" ");     
    }
    Serial.print(i);
    Serial.print(" - ");
    Serial.print(char('a' + i));   
    Serial.print(" # ");
    Serial.println(index_array[i]);    
  }
  Serial.println("");
#endif
    
  useraction();                                       // Benutzerfunktion, die das Replace-Array entsprechend den Paramtern modifizieren kann  
  
  String sResponse, sHeader;

  sResponse = getFile(pageDaten.Path);                // Daten der angeforderten Seite holen

  if (sResponse == "")                                // Seite nicht gefunden, 404 schicken
  {
    sHeader = F("HTTP/1.1 404 Not found\r\nContent-Length: ~response-length~\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
    sResponse = F("<html><head><title>404 Not Found</title></head><body><h1>Not Found</h1><p>The requested URL was not found on this server.</p></body></html>");
  }
  else
  {
    sHeader = F("HTTP/1.1 200 OK\r\nContent-Length: ~response-length~\r\nContent-Type: text/html\r\nConnection: close\r\n\r\n");
    ulReqcount++;                                     // Anzahl Verbindungen

    if (index_array)                                  // gibt es was zu ersetzen?
    {
      char temp[] = "~0~";                            // Replace-String bauen
      for (int i = 0; i < ANZ_REPLACE; i++)
      {
        temp[1] = i + 'a';                            // nächsten Buchstaben setzen -> ~a~ ... bis ANZ_REPLACE
        sResponse.replace(temp, index_array[i]);      // und Variablen wert aus Arry in html-Text schreiben
      }
    }
  }

  // Send the response to the client
  Serial.print("Laenge: ");
  Serial.println(sResponse.length());
  char buffer[7];
  itoa(sResponse.length(), buffer, 10);
  sHeader.replace("~response-length~", buffer);       // PayLoad Länge in Header eintragen
  client.print(sHeader);

  int datapos = 0;
  while (datapos < sResponse.length())                // senden bis alles raus, client.print buffer ist nur 2292 Byte? Wir senden nur 2000 Byte im Stück
  {
    client.print(sResponse.substring(datapos, datapos + 2000));
    datapos += 2000;   
  }
  client.stop();

  Serial.println("Client disonnected");
  Serial.print("Request: ");
  Serial.print(ulReqcount);
  Serial.print(" # Reconnect: ");
  Serial.println(ulReconncount);
  Serial.println("");
}

// ----------------------------------------------------------------------

void decodeParameter(String sRequest)
{
  pageDaten.Path = "";
  for (int i = 0; i < MAX_PARAMETER; i++)
  {
    pageDaten.Index[i] = "";
    pageDaten.Value[i] = "";
  }
  // get path; end of path is either space or ?
  // Syntax is e.g. GET /{filename}?pin=MOTOR1STOP HTTP/1.1
  
  String sParam = "", sGetstart = "GET ", sPoststart = "POST ";
  int iStart, iStartGet, iStartPost, iEnd, iEndSpace, iStartQuest, iEndQuest;
  iStartGet = sRequest.indexOf(sGetstart);                          // Position von 'GET ' im Puffer bestimmen
  if (iStartGet >= 0)
  {
    iStartGet += +sGetstart.length();                               // Zeiger auf Position hinter 'GET '
    iEndSpace = sRequest.indexOf(" ", iStartGet);
    iEndQuest = sRequest.indexOf("?", iStartGet);
    if (iEndQuest > 0)                                           // Fragezeichen gefunden, also Parameter
    {
      pageDaten.Path  = sRequest.substring(iStartGet, iEndQuest);   // Pfad extrahieren und merken
      sParam = sRequest.substring(iEndQuest, iEndSpace);         // Parameter extrahieren
      sParam.replace("?", "");                                   // Fragezeichen am Anfang entfernen
    }
    else
    {
      pageDaten.Path  = sRequest.substring(iStartGet, iEndSpace);    // keine Parameter, nur Pfad merken
    }
    if (pageDaten.Path == "/")                                      // wenn ohne Dateinamen dann index.html als Default
    {
      pageDaten.Path += "index.html";
    }
  }

  // are there parameters?
  if (sParam.length() > 0)                                         // Parameter gefunden
  {
    String temp;
    iStart = 0;
    iEndSpace = 0;                                              // Position von '&' -> neuer Parameter
    iEndQuest = 0;                                              // Position von '=' -> Trenner Name - Wert
    char anz = 0;                                               // Index für Parameter-Array
    while (iEndSpace != -1)                                     // was gefunden
    {
      if (sParam.length() > 0)                                  // überhaupt was da?
      {
        iEndSpace = sParam.indexOf("&", iStart);                // erstes & suchen
        iEndQuest = sParam.indexOf("=", iStart + 1);            // Suchen nach '=' ab Zeichen nach '&'
        if (iEndQuest > iStart)                                 // auch mit '=' drin?
        {
          pageDaten.Index[anz] = sParam.substring(iStart, iEndQuest);
          pageDaten.Value[anz] = sParam.substring(iEndQuest + 1, iEndSpace);
          pageDaten.Value[anz].trim();
          iStart = iEndSpace + 1;                               // nur Startposition hinter dem gefundenen '&'
          anz++;                                                // Arrayindex erhöhen
          if (anz == MAX_PARAMETER)
          {
            Serial.println("!!! Zuviel Parameter !!!");
            break;
          }
        }
        else
        {
          Serial.println("!!! Kein '=' gefunden !!!");
          break;
        }
      }
    }
  }
}

// ----------------------------------------------------------------------

String getFile(String filename)
{
  String temp;

  File tempFile = SPIFFS.open(filename, "r");
  if (tempFile)
  {
    temp = tempFile.readString();
    tempFile.close();
  }
  else
  {
    Serial.print("Failed to open ");
    Serial.println(filename);
  }
  return temp;
}


